# -*- Mode: Python -*-
# vi:si:et:sw=4:sts=4:ts=4

cdef extern from "squale-gil.h":
    # FIXME: when changing this, change the one at the bottom too
    ctypedef enum SqualeResultStatus:
        SQUALE_RESULT_ERROR = 0
        SQUALE_RESULT_AFFECTED_ROWS = 1
        SQUALE_RESULT_RESULTSET = 2
        SQUALE_RESULT_RESULTSET_WITH_WARNING = 3

    ctypedef struct SqualeResult:
        SqualeResultStatus status

        int assignation_time
        int processing_time

        int num_fields
        long num_rows

        char **column_names
        char ***data

        int affected_rows
        char *error
        char *warning

    SqualeResult *squale_run_query_with_gil(char *connection_name,
                                   char *query,
                                   char *socket_name,
                                   int s_timeout)
    int squale_free_result(SqualeResult *result)

class SqualeError(Exception):
    pass

cdef class Result:
    '''result object for Squale queries'''
    cdef SqualeResult *result
    def __new__(self, connection_name, query, socket_name='/tmp/squale.sock', s_timeout=5):
        self.result = squale_run_query_with_gil(connection_name, query, socket_name, s_timeout)
        if self.result.status == SQUALE_RESULT_ERROR:
            raise SqualeError(self.result.error)

    def __dealloc__(self):
        squale_free_result(self.result)

    def get_affected_rows(self):
        '''Get the number of affected rows.'''
        if self.result.status != SQUALE_RESULT_AFFECTED_ROWS:
            raise SqualeError("Result is not a result set with affected rows")
        return self.result.affected_rows

    def get_status(self):
        '''Get the status of the query.'''
        return self.result.status
        
    def get_column_names(self):
        '''Get the names of columns as a list.'''
        cdef int i
        cdef char * name
        result = []
        for i from 0 <= i < self.result.num_fields:
            name = self.result.column_names[i]
            result.append(name)
        return result
        
    def get_data(self):
        '''Get the result rows as a list of lists.'''
        cdef int i, j
        cdef char * value

        if self.result.status != SQUALE_RESULT_RESULTSET:
            raise SqualeError("Result is not a result set with data")

        data = []
        for i from 0 <= i < self.result.num_rows:
            row = []
            for j from 0 <= j < self.result.num_fields:
                if self.result.data[i][j]:
                    value = self.result.data[i][j]
                else:
                    value = ""
                row.append(value)
            data.append(row)

        return data

# nice to have an enum
# FIXME: would be nice to generate this automatically
RESULT_AFFECTED_ROWS = SQUALE_RESULT_AFFECTED_ROWS
RESULT_RESULTSET = SQUALE_RESULT_RESULTSET
RESULT_RESULTSET_WITH_WARNING = SQUALE_RESULT_RESULTSET_WITH_WARNING
